#!/bin/bash
#cifs-config.sh
#
# Copyright © 2008-2013 Novell, Inc.  All Rights Reserved.
#
# USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE DEVELOPER LICENSE
# AGREEMENT OR OTHER AGREEMENT THROUGH WHICH NOVELL, INC. MAKES THE WORK
# AVAILABLE. THIS WORK MAY NOT BE ADAPTED WITHOUT  NOVELL'S PRIOR WRITTEN
# CONSENT.
#
# NOVELL PROVIDES THE WORK "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY,
# INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY,FITNESS FOR
# A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  NOVELL, THE AUTHORS OF THE
# WORK, AND THE OWNERS OF COPYRIGHT IN THE WORK ARE NOT LIABLE FOR ANY
# CLAIM, DAMAGES, OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT, OR OTHERWISE, ARISING FROM, OUT OF, OR IN CONNECTION WITH THE WORK
# OR THE USE OR OTHER DEALINGS IN THE WORK.
#
# All rights reserved.

###########################################################################
#DESCRIPTION: cifs-config.sh script is used for configuring CIFS at 
#install time, configure proxy user and extend schema.
#It can be executed in two modes
#1. without any command line option for:
#   schema Extension or proxy user configuration
#2. With command line options for 
#   Updating config file, Proxy User config, and NMAS Method installation
#When called with command line options, it should be called as below
#with 10 options, last being list of user contexts separated by spaces
#cifs-config.sh <1-LDAP IP> <2-LDAP Port> <3-Admin DN> <4-Proxy DN> 
#				 <5-Server Context>  <6-SSL or No-SSL> <7-OCS or File>  
#				 <8-Enable/Disable Subtree-Search> 
#				 <9-Grant Righs to Proxy over user Contexts>
#				 <10-List of user contexts separated by spaces>
#DN Formats should be in LDAP format
#For Enable/Disable Search: 0 - disable, 1 - enable, Others - Don't change
#For Grant Rights to proxy: 0 - disable, 1 - enable, Others - Don't change
###########################################################################
host_name=`hostname`
TMP_LDIF_CONF_FILE="/opt/novell/cifs/schema/cifstmp.ldif"
TMP_LDIF_PRXYUSR="/tmp/cifsprxyusradd.ldif"
TMP_LDIF_TRUSTEE="/tmp/cifs_trustee.ldif"
NO_OF_ARGS=9

NMAS_ADD_BIN="/opt/novell/eDirectory/bin/nmasinst -addmethod"

LOG_FILE="/var/log/YaST2/cifsYast.log"
PROXY_LOG_FILE="/var/opt/novell/log/proxymgmt/pxymgmt.log"
YAST2_LOG_FILE="/var/log/YaST2/y2log"
CIFS_PROXY_POLICY="CIFS Default Policy"
cifs_conf_file="
#        Novell CIFS Server Configuration File
########################################################
### The following configurations are case sensitive. ###
########################################################
#-SERVERNAME  [Server Name]   ( Will be truncated to 16 characters )
#-UNICODE [yes/no]
#-AUTHENT [local/domain]
#-COMMENT [Server Comment]
#-WORKGROUP [workgroup]
#-AUDITING [yes/no]  Default set to no
#-GUEST [yes/no]  Default set to no
#-LM [0..5] Default set to 0(accepts both NTLMv1 and NTLMv2 hashes)

-SERVERNAME $host_name
-UNICODE yes
-AUTHENT local
-DIALECT SMB3
-WORKGROUP WORKGROUP
######################################################################################
### The Configuration further to this will be read every time the server comes up. ###
######################################################################################
-LOG_LEVEL error
-AUDITING no
-GUEST no
-FILE_FLUSH no
-LM 0
-OPLOCK_BREAK_ACK_TIMEOUT 30
-CHANGE_NOTIFY yes"

verify_ocs() {
currentUser=`/opt/novell/cifs/bin/retrieve_proxy_cred username`
currentPass=`/opt/novell/cifs/bin/retrieve_proxy_cred password`

if [ "$currentUser" == "$cifsProxyUser" ]
then
    if [ "$currentPass" == "$cifsPassword" ]
    then
       	ocs=1
    else
       	ocs=0
    fi

else
    ocs=0;
fi

}

#Create CIFS Proxy User, store credentials and grant rights to proxy user
config_proxyuser()
{
	cifsProxyUser="$1"
	logindn="$2"
	hostname="$3"
	ldapport=$4
	ssl=$5
	grantrightsoncontexts=$6

	sh /opt/novell/cifs/bin/cifs_create_proxy_user.sh "$cifsProxyUser" "$logindn" "$hostname" "$ldapport" "$ssl"
	if [ $? != 0 ] 
	then
		echo "`date +'%Y-%m-%d %H:%M:%S' ` : [CIFS]Failed to create Proxy User" >> $PROXY_LOG_FILE
		echo "`date +'%Y-%m-%d %H:%M:%S' ` : [CIFS]Failed to create Proxy User" >> $YAST2_LOG_FILE
		exit 1
	fi
    
	#Store proxy credentials in OCS or file
	sh /opt/novell/cifs/bin/cifs_update_proxy_cred.sh "$cifsProxyUser"
	if [ $? != 0 ]
	then
		echo "`date +'%Y-%m-%d %H:%M:%S' ` : [CIFS]Failed to store proxy user credential" >> $PROXY_LOG_FILE
		echo "`date +'%Y-%m-%d %H:%M:%S' ` : [CIFS]Failed to store proxy user credential" >> $YAST2_LOG_FILE
		exit 2
	fi
	        
	#Grant the rights for proxy user
	if [ $grantrightsoncontexts == 0 ] #Don't grant rights over user contexts
	then
		sh /opt/novell/cifs/bin/cifs_proxy_rights_assign.sh -d "$logindn" -u "$cifsProxyUser" -i "$hostname" -p "$ldapport" -s "$ssl" -r 1
		if [ $? != 0 ] 
		then
			echo "`date +'%Y-%m-%d %H:%M:%S' ` : [CIFS]Failed to grant rights to proxy user over NCP Server" >> $PROXY_LOG_FILE
			echo "`date +'%Y-%m-%d %H:%M:%S' ` : [CIFS]Failed to grant rights to proxy user over NCP Server" >> $YAST2_LOG_FILE
			exit 3
		fi
	elif [ $grantrightsoncontexts == 1 ] #Grant rights over user contexts
	then
		sh /opt/novell/cifs/bin/cifs_proxy_rights_assign.sh -d "$logindn" -u "$cifsProxyUser" -i "$hostname" -p "$ldapport" -s "$ssl" -r 3
		if [ $? != 0 ] 
		then
			echo "`date +'%Y-%m-%d %H:%M:%S' ` : [CIFS]Failed to grant rights to proxy user" >> $PROXY_LOG_FILE
			echo "`date +'%Y-%m-%d %H:%M:%S' ` : [CIFS]Failed to grant rights to proxy user" >> $YAST2_LOG_FILE
			exit 3
		fi
	fi
}


if [ $# -eq 0 ] #No arguments. Called from command line for schema extension or proxy user config
then
#Called from the CLI
	echo "    Choose the operations  "
	echo "1)  Extend Schema     "
	echo "2)  Configure Proxy User "
	echo "3)  Exit "
	read input

	if [ $input == 4 ]
	then
        	grep $host_name /etc/opt/novell/cifs/cifs.conf >>/dev/null 2>&1
		if [ $? -ne 0 ]
		then
	        echo "$cifs_conf_file" > /etc/opt/novell/cifs/cifs.conf
		else	
			echo "Configuration File already built."
		fi
	fi

	if [ $input == 1 ] #Schema Extension Operation
	then
		echo "*********************************"
		echo "*   Extending  Schema           *"	
		printf "*********************************\n\n"
		if [ -e "/opt/novell/oes-install/util/extend_schema" ]
		then
			echo "Extending the schema using the Oes-install utiliy - extend_schema" 
			echo "Please give input in the following manner "
			echo "<Username> <Password> <ServerName> "
			echo "e.g. cn=admin,o=novell novell 192.168.0.1 "
			printf "\n"
			while [ 1 ] 
			do
				read config 
				UserName=`echo $config |cut -f1 -s -d" " `
				Password=`echo $config |cut -f2 -d" " `
				ServerName=`echo $config |cut -f3 -d" " `
				if [ -z "$UserName" ] ||  [ -z "$Password" ] || [ -z "$ServerName" ] 
				then
					printf "Invalid Input !!"
					printf "\n\n"
				else
					retValue=`/opt/novell/oes-install/util/extend_schema $UserName $Password $ServerName /opt/novell/cifs/schema/nfap.ldif`;
					echo $retValue;
					break
				fi 
			
			done
		
		else # End of Schema extension option using OES Schema Extension utility
			echo "Extending the schema using the eDirectory utility -ndssch"
			echo "Please give input in the following manner "
			echo "<hostname>:<port>  <treename> <admin_fdn>"
			echo "e.g. 192.168.0.1:524  CIFS-LINUX_PREM_TREE admin.novell " 
			printf "\n"
			while [ 1 ] 
			do
				read config 
				Server=`echo $config |cut -f1 -s -d" " `
				TreeName=`echo $config |cut -f2 -d" " `
				UserName=`echo $config |cut -f3 -d" " `
				if [ -z "$Server" ] ||  [ -z "$TreeName" ] || [ -z "$UserName" ] 
				then
					printf "Invalid Input !!"
					printf "\n\n"
				else
					retValue=`/opt/novell/eDirectory/bin/ndssch -h $Server -t $TreeName $UserName /opt/novell/cifs/schema/nfap.sch`;
					echo $retValue;
					break
				fi 
			done #of while
		
		fi #End of schema extension using eDir schema extension utility ndssch
		
	else # End of Schema Extension. Input =1
	if [ $input == 2 ] #Configure Proxy user
	then
		
		echo "***************************************************************"
		echo "*   Adding the cifsProxy User To the NDS                      *"	
		echo "*   Same UserName and password will be stored in OCS          *"
		echo "***************************************************************"
		printf "\n"
		echo "<hostname> <ldapportnumber> <LoginDN> <Password> <cifsProxyUserDN> <cifsPassword> <ServerContext> <SSL> <OCS> <RightsOnContexts> "
		echo "  host name      = Ldap server name or IP address"
		echo "  ldapportnumber     = LDAP port to use - 636 is ldap ssl default"
		echo "  LoginDN        = User name to login as"
		echo "  password       = User password"
		echo "  cifsProxyUserDN= Full DN of CIFS Proxy User"
		echo "  cifsPassword   = CIFS Proxy User Password"
		echo "  serverContext  = Context of the local NCP Server "
		echo "  SSL            = 1 or 0 (if 1, SSL will be enabled)" 
		echo "  OCS 	       = 1 or 0 (if 1, OCS will be used to store the Proxy User Credentials ) "
		echo "  Grant rights to proxy = 1, 0 (if 1, proxy will be granted rights over user contexts)"
		echo "  eg  : 192.168.0.1 636 cn=admin,o=novell novell cn=cifsProxyUser,o=novell pass o=novell 1 1 1";
		echo "        or"
		echo "        192.168.0.1 389 cn=admin,o=novell novell cn=cifsProxyUser,o=novell pass o=novell 0 1 1"
			
		while [ 1 ]
		do
			
			read config
			CIFSCONF=/etc/opt/novell/cifs/cifs.conf			#File for storing the CIFS Server Configuration
			CIFSCTX=/etc/opt/novell/cifs/cifsctxs.conf  		#File for storing the CIFS User Context
			    
			#Split the input into parameters
			hostname=`echo $config |cut -f1 -s -d" " `
			port=`echo $config |cut -f2 -d" " `
			logindn=`echo $config |cut -f3 -d" " `
			password=`echo $config |cut -f4 -d" " `
			cifsProxyUser=`echo $config |cut -f5 -d" " `
			cifsPassword=`echo $config |cut -f6 -d" " `
			serverdn=`echo $config |cut -f7 -d" " `
			ssl=`echo $config |cut -f8 -d" " `
			ocs=`echo $config |cut -f9 -d" " `
			proxyrights=`echo $config |cut -f10 -d" " `
			if [ $ssl != 0 ]
			then
				ssl=1;		
			fi
			if [ -z "$hostname" ] ||  [ -z "$port" ] || [ -z "$logindn" ] || [ -z "$password" ] || [ -z "$cifsProxyUser" ] || [ -z "$cifsPassword" ] || [ -z "$serverdn" ]   
			then
				echo "Invalid Input "
			else
				#Now process the user contexts one by one
				echo "########################################################"> $CIFSCTX 
				echo "#             Novell CIFS User Context file            #">> $CIFSCTX
				echo "########################################################">> $CIFSCTX
				echo "###  Please do not modify this file, changes will be ignored.">> $CIFSCTX
				echo "###  Use iManager to edit the user contexts" >> $CIFSCTX
					
				usercontexts=`echo $config | cut -f11- -d" "`
				for ctx in $usercontexts
				do
					echo $ctx>>$CIFSCTX 
				done
				    
				echo "$cifs_conf_file" >  $CIFSCONF
				if [ $ocs == 1 ]
				then
					echo "-OCS yes"     >> $CIFSCONF
				else
					echo "-OCS no"     >> $CIFSCONF
				fi

				if [ $ssl == 1 ]
				then
					echo "-SSL  yes"      >> $CIFSCONF
				else
					echo "-SSL no"     >> $CIFSCONF
				fi
				echo "-LDAPPORT $port" >> $CIFSCONF
					
				#Configure Proxy user
				OES_DATA=$password
				OES_DATA2=$cifsPassword
				export OES_DATA
				export OES_DATA2
				config_proxyuser "$cifsProxyUser" "$logindn" "$hostname" "$port" "$ssl" "$usercontextrights" "$proxyrights"
				break;
			fi #with valid inputs
				
		done #of while(1)
			
	else #End of configure proxy option [ input = 2]
		exit 0
	fi #End of invalid options [input]
		
fi #End of executing cifs-config.sh without command line parameters

else #Called with command line parameters. Mostly from YaST
#REFER TO TOP OF THE FILE FOR USAGE DETAILS
	if [ $# -eq $NO_OF_ARGS ]
	then
		frmyast=1
		echo "`date +'%Y-%m-%d %H:%M:%S' ` : No user contexts configured " >> $LOG_FILE
		echo "`date +'%Y-%m-%d %H:%M:%S' ` : No user contexts configured" >> $YAST2_LOG_FILE
	elif [ $# -gt $NO_OF_ARGS ]
	then
		frmyast=1
	else
		frmyast=0
	fi

	#Called from the YAST
	CIFSCONF=/etc/opt/novell/cifs/cifs.conf		#File for storing the CIFS Server Configuration
	CIFSENC=/etc/opt/novell/cifs/.cifspwd.enc       #Local file where the Proxy User Credentials are stored encrypted if OCS is not used to store it.
	CIFSCTX=/etc/opt/novell/cifs/cifsctxs.conf 	#File for storing the CIFS User Context

	echo "`date +'%Y-%m-%d %H:%M:%S' ` : cifs-config.sh log" >> $LOG_FILE
	echo "`date +'%Y-%m-%d %H:%M:%S' ` : cifs-config.sh log" >> $YAST2_LOG_FILE

	echo "`date +'%Y-%m-%d %H:%M:%S' ` : Command = $*" >> $LOG_FILE
	echo "`date +'%Y-%m-%d %H:%M:%S' ` : Command = $*" >> $YAST2_LOG_FILE

	# Storing the passed configuration
	hostname=$1					#LDAP Server Address			
	port=$2						#LDAP Server Port
	logindn=$3					#Admin Credentials
	password=$OES_DATA				#Admin Password
	cifsProxyUser=$4				#Full DN of Proxy User
	cifsPassword=$OES_DATA2				#Proxy User Password
	serverContext=$5				#Server Context where local NCP Server is located
	ssl=$6						#Use SSL or not
	ocs=$7						#Use OCS or not
	subtreesearch=$8				#Enable/Disable Subtree search
	usercontextrights=$9				#Whether to grant rihts to proxy user over user contexts
	
	#Moving the current position to user contexts, 
	#So that the 1st element points to 1st context.
	shift 9
	
	# Storing the context into the context file
	
	if [ $frmyast == 1 ]
	then
		echo "########################################################"> $CIFSCTX 
		echo "#             Novell CIFS User Context file            #">> $CIFSCTX
		echo "########################################################">> $CIFSCTX
		echo "###  Please do not modify this file, changes will be ignored.">> $CIFSCTX
		echo "###  Use iManager to edit the user contexts" >> $CIFSCTX 	
	
		while [ -n "$1" ]
        	do
                	echo $1 >> $CIFSCTX
                	shift
        	done
	fi
	
	#Default configuration file settings
	echo "$cifs_conf_file" >  $CIFSCONF
	
	#Update cifs conf file with OCS, SSL, LDAP Port
	#and Subtree Search options
	if [ $ocs == 1 ]
	then
		echo "-OCS yes"     >> $CIFSCONF
	else
		echo "-OCS no"     >> $CIFSCONF
	fi
	
	if [ $ssl == 1 ] #ssl or not
	then
		echo "-SSL  yes"      >> $CIFSCONF
	else
		echo "-SSL no"     >> $CIFSCONF
	fi

	echo "-LDAPPORT $port" >> $CIFSCONF #LDAP port
	
	if [ $subtreesearch == 0 ]
	then
		echo "-SUBTREE no" >> $CIFSCONF
	elif [ $subtreesearch == 1 ]
	then
		echo "-SUBTREE yes" >> $CIFSCONF
	else
		echo "Invalid value passed for SUBTREE Search option" >> $PROXY_LOG_FILE
		echo "As default, SUBTREE Search is not enabled" >> $PROXY_LOG_FILE
		echo "Invalid value passed for SUBTREE Search option" 
		echo "As default, SUBTREE Search is not enabled" 
		echo "-SUBTREE no" >> $CIFSCONF
	fi
	    	
	#Configure proxy user
	export OES_DATA
	export OES_DATA2
	config_proxyuser "$cifsProxyUser" "$logindn" "$hostname" "$port" "$ssl" "$usercontextrights"

	#Install NMAS Method.
	#It requires the tree name. Same is retrieved from nds config file uisng ndsconfig tool
	treeName=`/opt/novell/eDirectory/bin/ndsconfig get n4u.base.tree-name | grep n4u.base.tree-name |  tr -d ' ' | awk -F= '{print $2}'`
	if test "$treeName" = ""
	then
		echo "[cifs-config.sh]: Treename is empty" >> $PROXY_LOG_FILE
		echo "[cifs-config.sh]: Treename is empty"
		exit 4
	fi
	echo "[CIFS]TreeName is: $treeName" >> $PROXY_LOG_FILE
	
	#Change the admin DN from LDAP format to eDir format
	dotlogindn=`echo "$logindn" | sed s/\,/\./g`	#Replace commas with dot
	dotlogindn=`echo $dotlogindn | tr '[:upper:]' '[:lower:]'`	#Convert to lower case
	dotlogindn=`echo $dotlogindn | sed 's/.st=/.s=/g'`	#Replace .st with .s as per NDS naming standard
    
	#Call nmasinst tool to install NMAS method into the tree
	ret=`$NMAS_ADD_BIN "$dotlogindn" "$treeName" "/opt/novell/cifs/share/nmasmthd/ntlm/config.txt" -w "$password" -checkversion`
	echo "`date +'%Y-%m-%d %H:%M:%S' ` : Return value of NMAS Add method is: $ret" >> $LOG_FILE
	echo "`date +'%Y-%m-%d %H:%M:%S' ` : [CIFS]Return value of NMAS Add method is: $ret" >> $PROXY_LOG_FILE
	echo "`date +'%Y-%m-%d %H:%M:%S' ` : [CIFS]Return value of NMAS Add method is: $ret" >> $YAST2_LOG_FILE
fi

#Starting novell-cifs if not already started
systemctl restart novell-cifs.service

exit 0



